@@ -5,15 +5,17 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1995-2003 Insecure.Com LLC. This       *
- * program is free software; you may redistribute and/or modify it under   *
- * the terms of the GNU General Public License as published by the Free    *
+ * The Nmap Security Scanner is (C) 1996-2004 Insecure.Com LLC. Nmap       *
+ * is also a registered trademark of Insecure.Com LLC.  This program is    *
+ * free software; you may redistribute and/or modify it under the          *
+ * terms of the GNU General Public License as published by the Free        *
  * Software Foundation; Version 2.  This guarantees your right to use,     *
  * modify, and redistribute this software under certain conditions.  If    *
  * you wish to embed Nmap technology into proprietary software, we may be  *
  * willing to sell alternative licenses (contact sales@insecure.com).      *
- * Many security scanner vendors already license Nmap technology such as   *
- * our remote OS fingerprinting database and code.                         *
+ * Many security scanner vendors already license Nmap technology such as  *
+ * our remote OS fingerprinting database and code, service/version         *
+ * detection system, and port scanning code.                               *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
@@ -23,8 +25,11 @@
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
  *   nmap-os-fingerprints or nmap-service-probes.                          *
- * o Executes Nmap                                                         *
- * o Integrates/includes/aggregates Nmap into an executable installer      *
+ * o Executes Nmap and parses the results (as opposed to typical shell or  *
+ *   execution-menu apps, which simply display raw Nmap output and so are  *
+ *   not derivative works.)                                                * 
+ * o Integrates/includes/aggregates Nmap into a proprietary executable     *
+ *   installer, such as those produced by InstallShield.                   *
  * o Links to a library or executes a program that does any of the above   *
  *                                                                         *
  * The term "Nmap" should be taken to also include any portions or derived *
@@ -51,8 +56,17 @@
  * the continued development of Nmap technology.  Please email             *
  * sales@insecure.com for further information.                             *
  *                                                                         *
+ * As a special exception to the GPL terms, Insecure.Com LLC grants        *
+ * permission to link the code of this program with any version of the     *
+ * OpenSSL library which is distributed under a license identical to that  *
+ * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * combinations including the two. You must obey the GNU GPL in all        *
+ * respects for all of the code used other than OpenSSL.  If you modify    *
+ * this file, you may extend this exception to your version of the file,   *
+ * but you are not obligated to do so.                                     *
+ *                                                                         *
  * If you received these files with a written license agreement or         *
- * contract stating terms other than the (GPL) terms above, then that      *
+ * contract stating terms other than the terms above, then that            *
  * alternative license agreement takes precedence over these comments.     *
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
@@ -78,11 +92,12 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html .                                  *
+ * http://www.gnu.org/copyleft/gpl.html , or in the COPYING file included  *
+ * with Nmap.                                                              *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: NmapOps.h,v 1.15 2003/09/13 04:19:54 fyodor Exp $ */
+/* $Id: NmapOps.h,v 1.22 2004/10/18 16:59:36 fyodor Exp $ */
 
 class NmapOps {
  public:
@@ -109,6 +124,12 @@ class NmapOps {
   const struct in_addr *v4sourceip();
   bool TCPScan(); /* Returns true if at least one chosen scan type is TCP */
   bool UDPScan(); /* Returns true if at least one chosen scan type is UDP */
+
+  /* Returns true if at least one chosen scan type uses raw packets.
+     Itdoes not currently cover cases such as TCP SYN ping scan which
+     can go either way based on whether the user is root or IPv6 is
+     being used.  It will return false in those cases where a RawScan
+     is not neccessarily used. */
   bool RawScan();
   void ValidateOptions(); /* Checks that the options given are
                              reasonable and consistant.  If they aren't, the
@@ -128,7 +149,7 @@ class NmapOps {
   int spoofsource; /* -S used */
   char device[64];
   int interactivemode;
-  int host_group_sz;
+  int ping_group_sz;
   int generate_random_ips; /* -iR option */
   FingerPrint **reference_FPs;
   u16 magic_port;
@@ -155,12 +176,25 @@ class NmapOps {
   void setMaxRttTimeout(int rtt);
   void setMinRttTimeout(int rtt);
   void setInitialRttTimeout(int rtt);
+
+  /* Similar functions for Host group size */
+  int minHostGroupSz() { return min_host_group_sz; }
+  int maxHostGroupSz() { return max_host_group_sz; }
+  void setMinHostGroupSz(unsigned int sz);
+  void setMaxHostGroupSz(unsigned int sz);
+  unsigned int maxTCPScanDelay() { return max_tcp_scan_delay; }
+  unsigned int maxUDPScanDelay() { return max_udp_scan_delay; }
+  void setMaxTCPScanDelay(unsigned int delayMS) { max_tcp_scan_delay = delayMS; }
+  void setMaxUDPScanDelay(unsigned int delayMS) { max_udp_scan_delay = delayMS; }
+
   int max_ips_to_scan; // Used for Random input (-iR) to specify how 
                        // many IPs to try before stopping. 0 means unlimited.
   int extra_payload_length; /* These two are for --data_length op */
   char *extra_payload;
   unsigned long host_timeout;
-  int scan_delay;
+  /* Delay between probes, in milliseconds */
+  unsigned int scan_delay;
+
   int scanflags; /* if not -1, this value should dictate the TCP flags
 		    for the core portscaning routine (eg to change a
 		    FIN scan into a PSH scan.  Sort of a hack, but can
@@ -177,7 +211,6 @@ class NmapOps {
   int osscan_guess;   /* Be more aggressive in guessing OS type */
   int numdecoys;
   int decoyturn;
-  int identscan;
   int osscan;
   int servicescan;
   int pingtype;
@@ -209,6 +242,10 @@ class NmapOps {
   int max_rtt_timeout;
   int min_rtt_timeout;
   int initial_rtt_timeout;
+  unsigned int max_tcp_scan_delay;
+  unsigned int max_udp_scan_delay;
+  unsigned int min_host_group_sz;
+  unsigned int max_host_group_sz;
   void Initialize();
   int addressfamily; /*  Address family:  AF_INET or AF_INET6 */  
   struct sockaddr_storage sourcesock;
