@@ -7,7 +7,7 @@
  *                                                                         *
  ***********************IMPORTANT NSOCK LICENSE TERMS***********************
  *                                                                         *
- * The nsock parallel socket event library is (C) 1999-2003 Insecure.Com   *
+ * The nsock parallel socket event library is (C) 1999-2004 Insecure.Com   *
  * LLC This library is free software; you may redistribute and/or          *
  * modify it under the terms of the GNU General Public License as          *
  * published by the Free Software Foundation; Version 2.  This guarantees  *
@@ -16,6 +16,15 @@
  * may be willing to sell alternative licenses (contact                    *
  * sales@insecure.com ).                                                   *
  *                                                                         *
+ * As a special exception to the GPL terms, Insecure.Com LLC grants        *
+ * permission to link the code of this program with any version of the     *
+ * OpenSSL library which is distributed under a license identical to that  *
+ * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * combinations including the two. You must obey the GNU GPL in all        *
+ * respects for all of the code used other than OpenSSL.  If you modify    *
+ * this file, you may extend this exception to your version of the file,   *
+ * but you are not obligated to do so.                                     *
+ *                                                                         * 
  * If you received these files with a written license agreement stating    *
  * terms other than the (GPL) terms above, then that alternative license   *
  * agreement takes precedence over this comment.                          *
@@ -47,7 +56,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nsock_ssl.c,v 1.5 2004/01/17 02:47:18 fyodor Exp $ */
+/* $Id: nsock_ssl.c,v 1.8 2004/08/29 09:12:05 fyodor Exp $ */
 
 
 #include "nsock.h"
@@ -71,7 +80,7 @@ void Nsock_SSL_Init() {
   if (sslnfo)
     return; /* Already done */
 
-  sslnfo = safe_zalloc(sizeof(*sslnfo));
+  sslnfo = (struct NsockSSLInfo *) safe_zalloc(sizeof(*sslnfo));
 
   SSL_load_error_strings();
   SSL_library_init();
