@@ -6,15 +6,17 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1995-2003 Insecure.Com LLC. This       *
- * program is free software; you may redistribute and/or modify it under   *
- * the terms of the GNU General Public License as published by the Free    *
+ * The Nmap Security Scanner is (C) 1996-2004 Insecure.Com LLC. Nmap       *
+ * is also a registered trademark of Insecure.Com LLC.  This program is    *
+ * free software; you may redistribute and/or modify it under the          *
+ * terms of the GNU General Public License as published by the Free        *
  * Software Foundation; Version 2.  This guarantees your right to use,     *
  * modify, and redistribute this software under certain conditions.  If    *
  * you wish to embed Nmap technology into proprietary software, we may be  *
  * willing to sell alternative licenses (contact sales@insecure.com).      *
- * Many security scanner vendors already license Nmap technology such as   *
- * our remote OS fingerprinting database and code.                         *
+ * Many security scanner vendors already license Nmap technology such as  *
+ * our remote OS fingerprinting database and code, service/version         *
+ * detection system, and port scanning code.                               *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
@@ -24,8 +26,11 @@
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
  *   nmap-os-fingerprints or nmap-service-probes.                          *
- * o Executes Nmap                                                         *
- * o Integrates/includes/aggregates Nmap into an executable installer      *
+ * o Executes Nmap and parses the results (as opposed to typical shell or  *
+ *   execution-menu apps, which simply display raw Nmap output and so are  *
+ *   not derivative works.)                                                * 
+ * o Integrates/includes/aggregates Nmap into a proprietary executable     *
+ *   installer, such as those produced by InstallShield.                   *
  * o Links to a library or executes a program that does any of the above   *
  *                                                                         *
  * The term "Nmap" should be taken to also include any portions or derived *
@@ -52,8 +57,17 @@
  * the continued development of Nmap technology.  Please email             *
  * sales@insecure.com for further information.                             *
  *                                                                         *
+ * As a special exception to the GPL terms, Insecure.Com LLC grants        *
+ * permission to link the code of this program with any version of the     *
+ * OpenSSL library which is distributed under a license identical to that  *
+ * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * combinations including the two. You must obey the GNU GPL in all        *
+ * respects for all of the code used other than OpenSSL.  If you modify    *
+ * this file, you may extend this exception to your version of the file,   *
+ * but you are not obligated to do so.                                     *
+ *                                                                         *
  * If you received these files with a written license agreement or         *
- * contract stating terms other than the (GPL) terms above, then that      *
+ * contract stating terms other than the terms above, then that            *
  * alternative license agreement takes precedence over these comments.     *
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
@@ -79,11 +93,12 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html .                                  *
+ * http://www.gnu.org/copyleft/gpl.html , or in the COPYING file included  *
+ * with Nmap.                                                              *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: osscan.cc,v 1.18 2003/09/11 02:12:57 fyodor Exp $ */
+/* $Id: osscan.cc,v 1.26 2004/10/12 09:34:11 fyodor Exp $ */
 
 
 #include "osscan.h"
@@ -146,7 +161,9 @@ int ossofttimeout, oshardtimeout;
 int seq_packets_sent = 0;
 int seq_response_num; /* response # for sequencing */
 double avg_ts_hz = 0.0; /* Avg. amount that timestamps incr. each second */
-if (target->timedout)
+struct link_header linkhdr;
+
+if (target->timedOut(NULL))
   return NULL;
 
 /* The seqs must start out as zero for the si struct */
@@ -163,6 +180,9 @@ get_random_bytes(&sequence_base, sizeof(unsigned int));
    pfatal("socket trobles in get_fingerprint");
  unblock_socket(rawsd);
  broadcast_socket(rawsd);
+#ifndef WIN32
+ sethdrinclude(rawsd);
+#endif
 
  /* Now for the pcap opening nonsense ... */
  /* Note that the snaplen is 152 = 64 byte max IPhdr + 24 byte max link_layer
@@ -174,7 +194,7 @@ oshardtimeout = MAX(500000, 5 * target->to.timeout);
 
  pd = my_pcap_open_live(target->device, /*650*/ 8192,  (o.spoofsource)? 1 : 0, (ossofttimeout + 500)/ 1000);
 
-if (o.debugging)
+if (o.debugging > 1)
    log_write(LOG_STDOUT, "Wait time is %dms\n", (ossofttimeout +500)/1000);
 
  flt_srchost = target->v4host().s_addr;
@@ -199,7 +219,7 @@ snprintf(filter, sizeof(filter), "dst host %s and (icmp or (tcp and src host %s)
  if ((tport = target->ports.nextPort(NULL, IPPROTO_TCP, PORT_CLOSED, false))) {
    closedport = tport->portno;
    target->FPR->osscan_closedtcpport = tport->portno;
- } else if ((tport = target->ports.nextPort(NULL, IPPROTO_TCP, PORT_UNFIREWALLED, false))) {
+ } else if ((tport = target->ports.nextPort(NULL, IPPROTO_TCP, PORT_UNFILTERED, false))) {
    /* Well, we will settle for unfiltered */
    closedport = tport->portno;
  } else {
@@ -207,7 +227,7 @@ snprintf(filter, sizeof(filter), "dst host %s and (icmp or (tcp and src host %s)
  }
 
 if (o.verbose && openport != (unsigned long) -1)
-  log_write(LOG_STDOUT, "For OSScan assuming that port %d is open and port %d is closed and neither are firewalled\n", openport, closedport);
+  log_write(LOG_STDOUT, "For OSScan assuming port %d is open, %d is closed, and neither are firewalled\n", openport, closedport);
 
  current_port = o.magic_port + NUM_SEQ_SAMPLES +1;
  
@@ -277,24 +297,21 @@ if (o.verbose && openport != (unsigned long) -1)
    timeout = 0;
 
    /* Insure we haven't overrun our allotted time ... */
-   if (o.host_timeout && (TIMEVAL_MSEC_SUBTRACT(t1, target->host_timeout) >= 0))
-     {
-       target->timedout = 1;
-       goto osscan_timedout;
-     }
-   while(( ip = (struct ip*) readip_pcap(pd, &bytes, oshardtimeout, NULL)) && !timeout) {
+   if (target->timedOut(&t1))
+     goto osscan_timedout;
+
+   while(( ip = (struct ip*) readip_pcap(pd, &bytes, oshardtimeout, NULL, &linkhdr)) && !timeout) {
      gettimeofday(&t2, NULL);
      if (TIMEVAL_SUBTRACT(t2,t1) > oshardtimeout) {
        timeout = 1;
      }
-     if (o.host_timeout && (TIMEVAL_MSEC_SUBTRACT(t2, target->host_timeout) >= 0))
-       {
-	 target->timedout = 1;
-	 goto osscan_timedout;
-       }
 
-     if (bytes < (4 * ip->ip_hl) + 4U)
+     if (target->timedOut(&t2))
+       goto osscan_timedout;
+
+     if (bytes < (4 * ip->ip_hl) + 4U || bytes < 20)
        continue;
+     setTargetMACIfAvailable(target, &linkhdr, ip, 0);
      if (ip->ip_p == IPPROTO_TCP) {
        tcp = ((struct tcphdr *) (((char *) ip) + 4 * ip->ip_hl));
        testno = ntohs(tcp->th_dport) - current_port + 1;
@@ -344,34 +361,38 @@ if (o.verbose && openport != (unsigned long) -1)
    seq_packets_sent = 0;
    while (seq_packets_sent < NUM_SEQ_SAMPLES) {
      if (o.scan_delay) enforce_scan_delay(NULL);
+     if (seq_packets_sent > 0) {
+       gettimeofday(&t1, NULL);
+       int remaining_us = 110000 - TIMEVAL_SUBTRACT(t1, seq_send_times[seq_packets_sent - 1]);
+       if (remaining_us > 0) {
+	 /* Need to spend at least .5 seconds in sending all packets to
+	    reliably detect 2HZ timestamp sequencing */
+	 usleep(remaining_us);
+       }
+     }
      send_tcp_raw_decoys(rawsd, target->v4hostip(), o.ttl, 
 			 o.magic_port + seq_packets_sent + 1, 
 			 openport, 
 			 sequence_base + seq_packets_sent + 1, 0, 
 			 TH_SYN, 0 , (u8 *) "\003\003\012\001\002\004\001\011\010\012\077\077\077\077\000\000\000\000\000\000" , 20, NULL, 0);
-     if (!o.scan_delay)
-       usleep( MAX(110000, target->to.srtt)); /* Main reason we wait so long is that we need to spend more than .5 seconds to detect 2HZ timestamp sequencing -- this also should make ISN sequencing more regular */
-   
      gettimeofday(&seq_send_times[seq_packets_sent], NULL);
+     t1 = seq_send_times[seq_packets_sent];
      seq_packets_sent++;
-     
+   
      /* Now we collect  the replies */
-     
      while(si->responses < seq_packets_sent && !timeout) {
        
        if (seq_packets_sent == NUM_SEQ_SAMPLES)
-	 ip = (struct ip*) readip_pcap(pd, &bytes, oshardtimeout, NULL);
-       else ip = (struct ip*) readip_pcap(pd, &bytes, 10, NULL);
+	 ip = (struct ip*) readip_pcap(pd, &bytes, oshardtimeout, NULL, &linkhdr);
+       else ip = (struct ip*) readip_pcap(pd, &bytes, 10, NULL, &linkhdr);
        
        gettimeofday(&t2, NULL);
        /*     error("DEBUG: got a response (len=%d):\n", bytes);  */
        /*     lamont_hdump((unsigned char *) ip, bytes); */
        /* Insure we haven't overrun our allotted time ... */
-       if (o.host_timeout && (TIMEVAL_MSEC_SUBTRACT(t2, target->host_timeout) >= 0))
-	 {
-	   target->timedout = 1;
-	   goto osscan_timedout;
-	 }
+       if (target->timedOut(&t2))
+	 goto osscan_timedout;
+
        if (!ip) { 
 	 if (seq_packets_sent < NUM_SEQ_SAMPLES)
 	   break;
@@ -387,12 +408,15 @@ if (o.verbose && openport != (unsigned long) -1)
        }
        lastipid = ip->ip_id;
 
-       if (bytes < (4 * ip->ip_hl) + 4U)
+       if (bytes < (4 * ip->ip_hl) + 4U || bytes < 20)
 	 continue;
+       setTargetMACIfAvailable(target, &linkhdr, ip, 0);
        if (ip->ip_p == IPPROTO_TCP) {
 	 /*       readtcppacket((char *) ip, ntohs(ip->ip_len));  */
 	 tcp = ((struct tcphdr *) (((char *) ip) + 4 * ip->ip_hl));
-	 if (ntohs(tcp->th_dport) < o.magic_port || ntohs(tcp->th_dport) - o.magic_port > NUM_SEQ_SAMPLES || ntohs(tcp->th_sport) != openport) {
+	 if (ntohs(tcp->th_dport) < o.magic_port || 
+	     ntohs(tcp->th_dport) - o.magic_port > NUM_SEQ_SAMPLES || 
+	     ntohs(tcp->th_sport) != openport) {
 	   continue;
 	 }
 	 if ((tcp->th_flags & TH_RST)) {
@@ -417,21 +441,24 @@ if (o.verbose && openport != (unsigned long) -1)
 	     }
 	     seq_response_num = si->responses;
 	   }
-	   si->responses++;
-	   si->seqs[seq_response_num] = ntohl(tcp->th_seq); /* TCP ISN */
-	   si->ipids[seq_response_num] = ntohs(ip->ip_id);
-	   if ((gettcpopt_ts(tcp, &timestamp, NULL) == 0))
-	     si->ts_seqclass = TS_SEQ_UNSUPPORTED;
-	   else {
-	     if (timestamp == 0) {
-	       si->ts_seqclass = TS_SEQ_ZERO;
+	   if (si->seqs[seq_response_num] == 0) {
+	     /* New response found! */
+	     si->responses++;
+	     si->seqs[seq_response_num] = ntohl(tcp->th_seq); /* TCP ISN */
+	     si->ipids[seq_response_num] = ntohs(ip->ip_id);
+	     if ((gettcpopt_ts(tcp, &timestamp, NULL) == 0))
+	       si->ts_seqclass = TS_SEQ_UNSUPPORTED;
+	     else {
+	       if (timestamp == 0) {
+		 si->ts_seqclass = TS_SEQ_ZERO;
+	       }
+	     }
+	     si->timestamps[seq_response_num] = timestamp;
+	     /*           printf("Response #%d -- ipid=%hu ts=%i\n", seq_response_num, ntohs(ip->ip_id), timestamp); */
+	     if (si->responses > 1) {
+	       seq_diffs[si->responses-2] = MOD_DIFF(ntohl(tcp->th_seq), si->seqs[si->responses-2]);
 	     }
 	   }
-	   si->timestamps[seq_response_num] = timestamp;
-	   /*           printf("Response #%d -- ipid=%hu ts=%i\n", seq_response_num, ntohs(ip->ip_id), timestamp); */
-	   if (si->responses > 1) {
-	     seq_diffs[si->responses-2] = MOD_DIFF(ntohl(tcp->th_seq), si->seqs[si->responses-2]);
-	   }      
 	 }
        }
      }
@@ -709,7 +736,7 @@ for(i=0; i < 9; i++) {
  if (last) FPtests[last]->next = NULL;
  
  osscan_timedout:
- if (target->timedout)
+ if (target->timedOut(NULL))
    FP = NULL;
  close(rawsd);
  pcap_close(pd);
@@ -1123,17 +1150,23 @@ int i;
 struct timeval now;
 double bestacc;
 int bestaccidx;
+ int starttimems = 0;
 
- if (target->timedout)
+ if (target->timedOut(NULL))
    return 1;
  
+ if (o.debugging > 2) {
+   starttimems = o.TimeSinceStartMS();
+   log_write(LOG_STDOUT|LOG_NORMAL|LOG_SKID, "Initiating OS Detection against %s at %.3fs\n", target->targetipstr(), starttimems / 1000.0);
+ }
+
  if (target->FPR == NULL)
    target->FPR = new FingerPrintResults;
 
  memset(si, 0, sizeof(si));
  if (target->ports.state_counts_tcp[PORT_OPEN] == 0 ||
      (target->ports.state_counts_tcp[PORT_CLOSED] == 0 &&
-      target->ports.state_counts_tcp[PORT_UNFIREWALLED] == 0)) {
+      target->ports.state_counts_tcp[PORT_UNFILTERED] == 0)) {
    if (o.osscan_limit) {
      if (o.verbose)
        log_write(LOG_STDOUT|LOG_NORMAL|LOG_SKID, "Skipping OS Scan due to absence of open (or perhaps closed) ports\n");
@@ -1144,17 +1177,12 @@ int bestaccidx;
  }
 
  for(itry=0; itry < 3; itry++) {
-   if (o.host_timeout) {   
-     gettimeofday(&now, NULL);
-     if (target->timedout || TIMEVAL_MSEC_SUBTRACT(now, target->host_timeout) >= 0)
-       {
-	 target->timedout = 1;
-	 return 1;
-       }
-   }
-   target->FPR->FPs[itry] = get_fingerprint(target, &si[itry]); 
-   if (target->timedout)
+   gettimeofday(&now, NULL);
+   if (target->timedOut(&now))
      return 1;
+
+   target->FPR->FPs[itry] = get_fingerprint(target, &si[itry]); 
+
    match_fingerprint(target->FPR->FPs[itry], &FP_matches[itry], 
 		     o.reference_FPs, OSSCAN_GUESS_THRESHOLD);
    if (FP_matches[itry].overall_results == OSSCAN_SUCCESS && 
@@ -1204,6 +1232,9 @@ int bestaccidx;
    match_fingerprint(target->FPR->FPs[target->FPR->goodFP], target->FPR, 
 		     o.reference_FPs, OSSCAN_GUESS_THRESHOLD);
 
+ if (o.debugging > 2) {
+   log_write(LOG_STDOUT|LOG_NORMAL|LOG_SKID, "Completed OS Detection against %s at %.3fs (took %.3fs)\n", target->targetipstr(), o.TimeSinceStartMS() / 1000.0, (o.TimeSinceStartMS() - starttimems) / 1000.0);
+ }
  
  return 1;
 }
@@ -1212,19 +1243,26 @@ int bestaccidx;
    top of a fingerprint.  Gives info which might be useful when the
    FPrint is submitted (eg Nmap version, etc).  Result is written (up
    to ostrlen) to the ostr var passed in */
-void WriteSInfo(char *ostr, int ostrlen, int openport, int closedport) {
+void WriteSInfo(char *ostr, int ostrlen, int openport, int closedport, 
+		const u8 *mac) {
   struct tm *ltime;
   time_t timep;
-  
+  char macbuf[16];
   timep = time(NULL);
   ltime = localtime(&timep);
 
-  snprintf(ostr, ostrlen, "SInfo(V=%s%%P=%s%%D=%d/%d%%Time=%X%%O=%d%%C=%d)\n", 
+  macbuf[0] = '\0';
+  if (mac)
+    snprintf(macbuf, sizeof(macbuf), "%%M=%02X%02X%02X", mac[0], mac[1], 
+	     mac[2]);
+
+  snprintf(ostr, ostrlen, "SInfo(V=%s%%P=%s%%D=%d/%d%%Tm=%X%%O=%d%%C=%d%s)\n", 
 	   NMAP_VERSION, NMAP_PLATFORM, ltime->tm_mon + 1, ltime->tm_mday, 
-	   (int) timep, openport, closedport);
+	   (int) timep, openport, closedport, macbuf);
 }
 
-char *mergeFPs(FingerPrint *FPs[], int numFPs, int openport, int closedport) {
+char *mergeFPs(FingerPrint *FPs[], int numFPs, int openport, int closedport, 
+	       const u8 *mac) {
 static char str[10240];
 struct AVal *AV;
 FingerPrint *currentFPs[32];
@@ -1244,7 +1282,7 @@ for(i=0; i < numFPs; i++) {
 }
 
 /* Lets start by writing the fake "Info" test for submitting fingerprints */
- WriteSInfo(str, sizeof(str), openport, closedport); 
+ WriteSInfo(str, sizeof(str), openport, closedport, mac); 
  p = p + strlen(str);
 
 do {
@@ -1517,6 +1555,7 @@ char *p, *q; /* OH YEAH!!!! */
  FPs = (FingerPrint **) safe_zalloc(sizeof(FingerPrint *) * max_records); 
 
  fp = fopen(fname, "r");
+ if (!fp) fatal("Unable to open Nmap fingerprint file: %s", fname);
 
  top:
 while(fgets(line, sizeof(line), fp)) {  
@@ -1713,7 +1752,7 @@ sethdrinclude(sd);
 realcheck = in_cksum((unsigned short *)pseudo, 20 /* pseudo + UDP headers */ +
  datalen);
 #if STUPID_SOLARIS_CHECKSUM_BUG
- udp->uh_sum = sizeof(struct udphdr_bsd) + datalen;
+ udp->uh_sum = sizeof(udphdr_bsd) + datalen;
 #else
 udp->uh_sum = realcheck;
 #endif
@@ -1849,7 +1888,7 @@ current_testno++;
 
 /* This next test doesn't work on Solaris because the lamers
    overwrite our ip_id */
-#if !defined(SOLARIS) && !defined(SUNOS) && !defined(IRIX)
+#if !defined(SOLARIS) && !defined(SUNOS) && !defined(IRIX) && !defined(HPUX)
 
 #ifdef WIN32
 if(!winip_corruption_possible()) {
