@@ -6,15 +6,17 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1995-2003 Insecure.Com LLC. This       *
- * program is free software; you may redistribute and/or modify it under   *
- * the terms of the GNU General Public License as published by the Free    *
+ * The Nmap Security Scanner is (C) 1996-2004 Insecure.Com LLC. Nmap       *
+ * is also a registered trademark of Insecure.Com LLC.  This program is    *
+ * free software; you may redistribute and/or modify it under the          *
+ * terms of the GNU General Public License as published by the Free        *
  * Software Foundation; Version 2.  This guarantees your right to use,     *
  * modify, and redistribute this software under certain conditions.  If    *
  * you wish to embed Nmap technology into proprietary software, we may be  *
  * willing to sell alternative licenses (contact sales@insecure.com).      *
- * Many security scanner vendors already license Nmap technology such as   *
- * our remote OS fingerprinting database and code.                         *
+ * Many security scanner vendors already license Nmap technology such as  *
+ * our remote OS fingerprinting database and code, service/version         *
+ * detection system, and port scanning code.                               *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
@@ -24,8 +26,11 @@
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
  *   nmap-os-fingerprints or nmap-service-probes.                          *
- * o Executes Nmap                                                         *
- * o Integrates/includes/aggregates Nmap into an executable installer      *
+ * o Executes Nmap and parses the results (as opposed to typical shell or  *
+ *   execution-menu apps, which simply display raw Nmap output and so are  *
+ *   not derivative works.)                                                * 
+ * o Integrates/includes/aggregates Nmap into a proprietary executable     *
+ *   installer, such as those produced by InstallShield.                   *
  * o Links to a library or executes a program that does any of the above   *
  *                                                                         *
  * The term "Nmap" should be taken to also include any portions or derived *
@@ -52,8 +57,17 @@
  * the continued development of Nmap technology.  Please email             *
  * sales@insecure.com for further information.                             *
  *                                                                         *
+ * As a special exception to the GPL terms, Insecure.Com LLC grants        *
+ * permission to link the code of this program with any version of the     *
+ * OpenSSL library which is distributed under a license identical to that  *
+ * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * combinations including the two. You must obey the GNU GPL in all        *
+ * respects for all of the code used other than OpenSSL.  If you modify    *
+ * this file, you may extend this exception to your version of the file,   *
+ * but you are not obligated to do so.                                     *
+ *                                                                         *
  * If you received these files with a written license agreement or         *
- * contract stating terms other than the (GPL) terms above, then that      *
+ * contract stating terms other than the terms above, then that            *
  * alternative license agreement takes precedence over these comments.     *
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
@@ -79,11 +93,12 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html .                                  *
+ * http://www.gnu.org/copyleft/gpl.html , or in the COPYING file included  *
+ * with Nmap.                                                              *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: tcpip.h,v 1.54 2003/09/20 09:03:00 fyodor Exp $ */
+/* $Id: tcpip.h,v 1.64 2004/08/29 09:12:04 fyodor Exp $ */
 
 
 #ifndef TCPIP_H
@@ -216,15 +231,15 @@ extern "C" {
 }
 #endif
 
-#include <setjmp.h>
-#include <errno.h>
-#include <signal.h>
-
 #if HAVE_SYS_SOCKIO_H
 #include <sys/sockio.h>  /* SIOCGIFCONF for Solaris */
 #endif
 #endif /* WIN32 */
 
+#include <setjmp.h>
+#include <errno.h>
+#include <signal.h>
+
 #include "nmap_error.h"
 #include "utils.h"
 #include "nmap.h"
@@ -272,11 +287,27 @@ class PacketTrace {
      call. */
   static void trace(pdirection pdir, const u8 *packet, u32 len,
 		    struct timeval *now=NULL);
+/* Adds a trace entry when a connect() is attempted if packet tracing
+   is enabled.  Pass IPPROTO_TCP or IPPROTO_UDP as the protocol.  The
+   sock may be a sockaddr_in or sockaddr_in6.  The return code of
+   connect is passed in connectrc.  If the return code is -1, get the
+   errno and pass that as connect_errno. */
+  static void traceConnect(u8 proto, const struct sockaddr *sock, 
+			   int socklen, int connectrc, int connect_errno,
+			   const struct timeval *now);
+};
+
+#define MAX_LINK_HEADERSZ 24
+struct link_header {
+  int datalinktype; /* pcap_datalink(), such as DLT_EN10MB */
+  int headerlen; /* 0 if header was too big or unavailaable */
+  u8 header[MAX_LINK_HEADERSZ];
 };
 
 struct interface_info {
-    char name[64];
-    struct in_addr addr;
+  char name[64];
+  struct in_addr addr;
+  struct in_addr netmask;
 };
 
 #ifndef HAVE_STRUCT_IP
@@ -404,6 +435,31 @@ struct icmp
 };
 #endif /* HAVE_STRUCT_ICMP */
 
+/* Represents a single probe packet, such as a SYN to port 80 or an
+   ICMP netmask request packet. Values are still in network byte order. */
+class IPProbe {
+ public:
+  IPProbe();
+  ~IPProbe();
+/* Takes an IP packet and stores _a copy_ of it, in this Probe,
+   adjusting proper header pointers and such */
+  int storePacket(u8 *ippacket, u32 len);
+  u32 packetbuflen; /* Length of the whole packet */
+  u8 *packetbuf; /* The packet itself */
+  struct ip *ipv4; /* IP header of packet */
+  struct icmp *icmp; /* icmp, tcp, and udp are NULL if the packet has no such header */
+  struct tcphdr *tcp;
+  udphdr_bsd *udp;
+
+  u8 af; /* AF_INET or AF_INET6 */
+  /* Resets everything to NULL.  Frees packetbuf if it is filled.  You
+     can reuse a Probe by calling Reset() and then a new
+     storePacket(). */
+  void Reset(); 
+ private:
+
+};
+
  /* This ideally should be a port that isn't in use for any protocol on our machine or on the target */
 #define MAGIC_PORT 49724
 #define TVAL2LONG(X)  X.tv_sec * 1e6 + X.tv_usec
@@ -448,6 +504,41 @@ int send_udp_raw( int sd, struct in_addr *source, const struct in_addr *victim,
 int send_ip_raw( int sd, struct in_addr *source, const struct in_addr *victim, 
 		 int ttl, u8 proto, char *data, u16 datalen);
 
+/* Builds a TCP packet (including an IP header) by packing the fields
+   with the given information.  It allocates a new buffer to store the
+   packet contents, and then returns that buffer.  The packet is not
+   actually sent by this function.  Caller must delete the buffer when
+   finished with the packet.  The packet length is returned in
+   packetlen, which must be a valid int pointer. */
+u8 *build_tcp_raw(const struct in_addr *source, 
+		  const struct in_addr *victim, int ttl, 
+		  u16 ipid, u16 sport, u16 dport, u32 seq, u32 ack, u8 flags,
+		  u16 window, u8 *options, int optlen, char *data, 
+		  u16 datalen, u32 *packetlen);
+
+/* Builds a UDP packet (including an IP header) by packing the fields
+   with the given information.  It allocates a new buffer to store the
+   packet contents, and then returns that buffer.  The packet is not
+   actually sent by this function.  Caller must delete the buffer when
+   finished with the packet.  The packet length is returned in
+   packetlen, which must be a valid int pointer. */
+u8 *build_udp_raw(struct in_addr *source, const struct in_addr *victim,
+ 		  int ttl, u16 sport, u16 dport, u16 ipid, char *data, 
+		  u16 datalen, u32 *packetlen);
+
+/* Builds an IP packet (including an IP header) by packing the fields
+   with the given information.  It allocates a new buffer to store the
+   packet contents, and then returns that buffer.  The packet is not
+   actually sent by this function.  Caller must delete the buffer when
+   finished with the packet.  The packet length is returned in
+   packetlen, which must be a valid int pointer. */
+u8 *build_ip_raw(struct in_addr *source, const struct in_addr *victim, 
+		 int ttl, u8 proto, u16 ipid, char *data, u16 datalen, 
+		 u32 *packetlen);
+
+/* Send a pre-built IPv4 packet */
+int send_ip_packet(int sd, u8 *packet, unsigned int packetlen);
+
 /* Much of this is swiped from my send_tcp_raw function above, which 
    doesn't support fragmentation */
 int send_small_fragz(int sd, struct in_addr *source, 
@@ -472,7 +563,10 @@ int send_ip_raw_decoys( int sd, const struct in_addr *victim, int ttl, u8 proto,
 /* Calls pcap_open_live and spits out an error (and quits) if the call fails.
    So a valid pcap_t will always be returned. */
 pcap_t *my_pcap_open_live(char *device, int snaplen, int promisc, int to_ms);
-
+// Returns whether the packet receive time value obtaned from libpcap
+// (and thus by readip_pcap()) should be considered valid.  When
+// invalid (Windows and Amiga), readip_pcap returns the time you called it.
+bool pcap_recv_timeval_valid();
 
 /* Returns a buffer of ASCII information about a packet that may look
    like "TCP 127.0.0.1:50923 > 127.0.0.1:3 S ttl=61 id=39516 iplen=40
@@ -493,12 +587,33 @@ int ipaddr2devname( char *dev, const struct in_addr *addr );
 int devname2ipaddr(char *dev, struct in_addr *addr);
 /* Where the above 2 functions get their info */
 struct interface_info *getinterfaces(int *howmany);
+/* Check whether an IP address appears to be directly connected to an
+   interface on the computer (e.g. on the same ethernet network rather
+   than having to route).  Returns 1 if yes, -1 if maybe, 0 if not. */
+int IPisDirectlyConnected(struct sockaddr_storage *ss, size_t ss_len);
 void sethdrinclude(int sd);
 int getsourceip(struct in_addr *src, const struct in_addr * const dst);
 /* Get the source IP and interface name that a packet
    to dst should be sent to.  Interface name is dynamically
    assigned and thus should be freed */
 char *getsourceif(struct in_addr *src, struct in_addr *dst);
+
+/* This function tries to determine the target's ethernet MAC address
+   from a received packet as follows:
+   1) If linkhdr is an ethernet header, grab the src mac (otherwise give up)
+   2) If overwrite is 0 and a MAC is already set for this target, give up.
+   3) If the packet source address is not the target, give up.
+   4) Use the routing table to try to determine rather target is
+      directly connected to the src host running Nmap.  If it is, set the MAC.
+
+   This function returns 0 if it ends up setting the MAC, nonzero otherwise
+
+   This function assumes that ip has already been verified as
+   containing a complete IP header (or at least the first 20 bytes).
+*/  
+
+int setTargetMACIfAvailable(Target *target, struct link_header *linkhdr,
+			    struct ip *ip, int overwrite);
 int islocalhost(const struct in_addr * const addr);
 int unblock_socket(int sd);
 int Sendto(char *functionname, int sd, const unsigned char *packet, int len, 
@@ -512,7 +627,12 @@ int Sendto(char *functionname, int sd, const unsigned char *packet, int len,
 const char *proto2ascii(u8 proto, bool uppercase=false);
 /* Hex dump */
 int get_link_offset(char *device);
-char *readip_pcap(pcap_t *pd, unsigned int *len, long to_usec, struct timeval *rcvdtime);
+/* If rcvdtime is non-null and a packet is returned, rcvd will be
+   filled with the time that packet was captured from the wire by
+   pcap.  If linknfo is not NULL, lnknfo->headerlen and
+   lnkinfo->header will be filled with the appropriate values. */
+char *readip_pcap(pcap_t *pd, unsigned int *len, long to_usec, 
+		  struct timeval *rcvdtime, struct link_header *linknfo);
 #ifndef HAVE_INET_ATON
 int inet_aton(register const char *, struct in_addr *);
 #endif
@@ -545,8 +665,10 @@ void broadcast_socket(int sd);
 /* Do a receive (recv()) on a socket and stick the results (upt to
    len) into buf .  Give up after 'seconds'.  Returns the number of
    bytes read (or -1 in the case of an error.  It only does one recv
-   (it will not keep going until len bytes are read */
-int recvtime(int sd, char *buf, int len, int seconds);
+   (it will not keep going until len bytes are read).  If timedout is
+   not NULL, it will be set to zero (no timeout occured) or 1 (it
+   did). */
+int recvtime(int sd, char *buf, int len, int seconds, int *timedout);
 
 /* This attempts to calculate the round trip time (rtt) to a host by timing a
    connect() to a port which isn't listening.  A better approach is to time a
@@ -561,6 +683,9 @@ typedef int (*PFILTERFN)(const char *packet, unsigned int len); /* 1 to keep */
 void set_pcap_filter(Target *target, pcap_t *pd, PFILTERFN filter, char *bpf, ...);
 #endif
 
+/* Just accept everything ... TODO: Need a better approach than this flt_ 
+   stuff */
+int flt_all(const char *packet, unsigned int len);
 int flt_icmptcp(const char *packet, unsigned int len);
 int flt_icmptcp_2port(const char *packet, unsigned int len);
 int flt_icmptcp_5port(const char *packet, unsigned int len);
